==> searchlogit-0.3.71-py3-none-any.whl: searchlogit/misc.py <==
"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
MISCELLANEOUS CODES
"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

import numpy as np

''' -------------------------------------------------------------------- '''
''' Function. Create a list with 'val' replicated n times                '''
''' -------------------------------------------------------------------- '''
def make_list(val, n):
    return [val] * n

def list_of_zeros(n): # {
    return make_list(0, n)
# }

###########################################################################
# CODES COMMON TO LATENT_CLASS AND LATENT_CLASS_MIXED
###########################################################################

''' -------------------------------------------------------------------- '''
''' Function. Define as [None, None, ..., None] where |[...]| = num_classes '''
''' -------------------------------------------------------------------- '''
def initialise_avail_latent(avail_latent, num_classes):
#{
    avail_latent = np.repeat(None, num_classes) if avail_latent is None else avail_latent
    return avail_latent
# }

''' -------------------------------------------------------------------- '''
''' Function. Initialise fit_intercept flag                              '''
''' -------------------------------------------------------------------- '''
def initialise_fit_intercept(params):
# {
    return '_inter' in np.concatenate(params)
# }

''' -------------------------------------------------------------------- '''
''' Function. Initialise class_params_spec                               '''
''' If unspecified, default to using all varnames in each latent class   '''
''' -------------------------------------------------------------------- '''
def initialise_class_params_spec(class_params_spec, isvars, varnames, num_classes):
# {
    if class_params_spec is None: # {
        if isvars is not None:
            class_vars = [var for var in varnames if var not in isvars]
        else:
            class_vars = varnames

        # Replicate class_vars and vstack the replicates
        class_params_spec = np.vstack(make_list(class_vars, num_classes))
    # }
    return class_params_spec
# }

''' -------------------------------------------------------------------- '''
''' Function.                                                            '''
''' -------------------------------------------------------------------- '''
def initialise_membership_as_probability(member_params_spec):
# {
    membership_as_probability = True if member_params_spec is True else False
    return membership_as_probability
# }

''' -------------------------------------------------------------------- '''
''' Function. Initialise member_params_spec                              '''
''' -------------------------------------------------------------------- '''
def initialise_member_params_spec(membership_as_probability, member_params_spec, isvars, varnames, num_classes):
# {
    if membership_as_probability:
        member_params_spec = np.vstack(make_list('dummy', num_classes-1))  # Replicate 'dummy' and vstack the replicates

    if member_params_spec is None:
    # {
        if isvars is not None: member_vars = isvars
        else: member_vars = varnames

        # Replicate member_vars and vstack the replicates
        member_params_spec = np.vstack(make_list(member_vars, num_classes-1))
    # }
    return member_params_spec
# }

''' -------------------------------------------------------------------- '''
''' Function. Initialise intercept_opts                                  '''
''' -------------------------------------------------------------------- '''
def initialise_opts(intercept_opts, num_classes):
# {
    if intercept_opts is None:
        intercept_opts = {}
    else:
    # {
        if 'class_intercept_alts' in intercept_opts:  # {
            if len(intercept_opts['class_intercept_alts']) != num_classes:
                raise ValueError("The key class_intercept_alts in intercept_opts must be the same length as num_classes")
    # }
    return intercept_opts
# }
''' -------------------------------------------------------------------- '''
''' Function. Setup multinomial_logit or mixed_logit                     '''
''' -------------------------------------------------------------------- '''
def setup_logit(i, logit, X, y, varnames, class_params_spec, avail, alts, transvars, gtol, mixed=False,
            seed=None, panels=None, randvars=None, correlation=None, n_draws=None, mnl_init=None):
# {
    varnames_i, fit_intercept_i = class_params_spec[i], False
    if '_inter' in varnames_i:  # {
        fit_intercept_i = True
        varnames_i = [var for var in varnames_i if var != '_inter']
    # }

    X_i_idx = [ii for ii, name in enumerate(varnames) if name in varnames_i]
    X_i = np.array(X)[:, X_i_idx]

    transvars_i = []
    if transvars is not None:
        transvars_i = [transvar for transvar in transvars if transvar in varnames_i]

    seed_i = seed + i if seed else None

    if mixed:
    # {
        randvars_i = {k: v for k, v in randvars.items() if k in varnames_i}
        correlation_i = correlation
        if correlation is not None and isinstance(correlation, list):
            correlation_i = [corvar for corvar in correlation if corvar in varnames_i]

        logit.setup(X_i, y, varnames_i, alts, avail=avail, transvars=transvars_i,
                             randvars=randvars_i, panels=panels, seed=seed_i,
                             fit_intercept=fit_intercept_i, correlation=correlation_i, n_draws=n_draws,
                             verbose=0, mnl_init=mnl_init, gtol=gtol)
    # }
    else:
        logit.setup(X_i, y, varnames_i, alts, avail=avail, transvars=transvars_i,
              fit_intercept=fit_intercept_i, seed = seed, verbose=0, gtol=gtol)
    logit.fit()
    return logit
# }

''' -------------------------------------------------------------------- '''
''' Function.                                                            '''
''' -------------------------------------------------------------------- '''
def revise_betas(i, logit, betas, intercept_opts, alts):
# {
    coefs = logit.coeff_est + np.random.normal(0, 0.01, len(logit.coeff_est))  # Add some random noise
    if 'class_intercept_alts' in intercept_opts:
    # {
        intercept_idx = np.array(intercept_opts['class_intercept_alts'][i]) - 2
        J = len(np.unique(alts))  # Code shortcut
        coefs = np.concatenate((coefs[intercept_idx], coefs[(J - 1):]))
    # }
    betas[i] = coefs + np.random.normal(0, 0.01, len(coefs))  # Add small amounts of noise
    return betas
# }

==> searchlogit-0.3.77-py3-none-any.whl: searchlogit/misc.py <==
"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
MISCELLANEOUS CODES
"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

import numpy as np

''' -------------------------------------------------------------------- '''
''' Function. Create a list with 'val' replicated n times                '''
''' -------------------------------------------------------------------- '''
def make_list(val, n):
    return [val] * n

def list_of_zeros(n): # {
    return make_list(0, n)
# }

###########################################################################
# CODES COMMON TO LATENT_CLASS AND LATENT_CLASS_MIXED
###########################################################################

''' -------------------------------------------------------------------- '''
''' Function. Define as [None, None, ..., None] where |[...]| = num_classes '''
''' -------------------------------------------------------------------- '''
def initialise_avail_latent(avail_latent, num_classes):
#{
    avail_latent = np.repeat(None, num_classes) if avail_latent is None else avail_latent
    return avail_latent
# }

''' -------------------------------------------------------------------- '''
''' Function. Initialise fit_intercept flag                              '''
''' -------------------------------------------------------------------- '''
def initialise_fit_intercept(params, fit_inter = None):
# {
    if fit_inter is not None:
        return '_inter' in np.concatenate(params)
    else:
        return  False

''' -------------------------------------------------------------------- '''
''' Function. Initialise class_params_spec                               '''
''' If unspecified, default to using all varnames in each latent class   '''
''' -------------------------------------------------------------------- '''
def initialise_class_params_spec(class_params_spec, isvars, varnames, num_classes):
# {
    if class_params_spec is None: # {
        if isvars is not None:
            class_vars = [var for var in varnames if var not in isvars]
        else:
            class_vars = varnames

        # Replicate class_vars and vstack the replicates
        class_params_spec = np.vstack(make_list(class_vars, num_classes))
    # }
    return class_params_spec
# }

''' -------------------------------------------------------------------- '''
''' Function.                                                            '''
''' -------------------------------------------------------------------- '''
def initialise_membership_as_probability(member_params_spec):
# {
    membership_as_probability = True if member_params_spec is True else False
    return membership_as_probability
# }

''' -------------------------------------------------------------------- '''
''' Function. Initialise member_params_spec                              '''
''' -------------------------------------------------------------------- '''
def initialise_member_params_spec(membership_as_probability, member_params_spec, isvars, varnames, num_classes):
# {
    if membership_as_probability:
        member_params_spec = np.vstack(make_list('dummy', num_classes-1))  # Replicate 'dummy' and vstack the replicates

    if member_params_spec is None:
    # {
        if isvars is not None: member_vars = isvars
        else: member_vars = varnames

        # Replicate member_vars and vstack the replicates
        member_params_spec = np.vstack(make_list(member_vars, num_classes-1))
    # }
    return member_params_spec
# }

''' -------------------------------------------------------------------- '''
''' Function. Initialise intercept_opts                                  '''
''' -------------------------------------------------------------------- '''
def initialise_opts(intercept_opts, num_classes):
# {
    if intercept_opts is None:
        intercept_opts = {}
    else:
    # {
        if 'class_intercept_alts' in intercept_opts:  # {
            if len(intercept_opts['class_intercept_alts']) != num_classes:
                raise ValueError("The key class_intercept_alts in intercept_opts must be the same length as num_classes")
    # }
    return intercept_opts
# }
''' -------------------------------------------------------------------- '''
''' Function. Setup multinomial_logit or mixed_logit                     '''
''' -------------------------------------------------------------------- '''
def setup_logit(i, logit, X, y, varnames, class_params_spec, avail, alts, transvars, gtol,
    mixed=False, panels=None, randvars=None, correlation=None, n_draws=None, mnl_init=None):
# {
    varnames_i, fit_intercept_i = class_params_spec[i], False
    if '_inter' in varnames_i:  # {
        fit_intercept_i = True
        varnames_i = [var for var in varnames_i if var != '_inter']
    # }

    X_i_idx = [ii for ii, name in enumerate(varnames) if name in varnames_i]
    X_i = np.array(X)[:, X_i_idx]

    transvars_i = []
    if transvars is not None:
        transvars_i = [transvar for transvar in transvars if transvar in varnames_i]

    if mixed:
    # {
        randvars_i = {k: v for k, v in randvars.items() if k in varnames_i}
        correlation_i = correlation
        if correlation is not None and isinstance(correlation, list):
            correlation_i = [corvar for corvar in correlation if corvar in varnames_i]

        logit.setup(X_i, y, varnames_i, alts, avail=avail, transvars=transvars_i,
                randvars=randvars_i, panels=panels, fit_intercept=fit_intercept_i,
                correlation=correlation_i, n_draws=n_draws, verbose=0, mnl_init=mnl_init, gtol=gtol)
    # }
    else:
        logit.setup(X_i, y, varnames_i, alts, avail=avail, transvars=transvars_i,
              fit_intercept=fit_intercept_i, verbose=0, gtol=gtol)
    logit.fit()
    return logit
# }

''' -------------------------------------------------------------------- '''
''' Function.                                                            '''
''' -------------------------------------------------------------------- '''
def revise_betas(i, logit, betas, intercept_opts, alts):
# {
    coefs = logit.coeff_est + np.random.normal(0, 0.01, len(logit.coeff_est))  # Add some random noise
    if 'class_intercept_alts' in intercept_opts:
    # {
        intercept_idx = np.array(intercept_opts['class_intercept_alts'][i]) - 2
        J = len(np.unique(alts))  # Code shortcut
        coefs = np.concatenate((coefs[intercept_idx], coefs[(J - 1):]))
    # }
    betas[i] = coefs + np.random.normal(0, 0.01, len(coefs))  # Add small amounts of noise
    return betas
# }
